@@ -4,11 +4,6 @@
 """
 This module contains the Python API for interacting with the Ledger API.
 """
-from ast import literal_eval
-from configparser import ConfigParser
-from pathlib import Path
-
-import pkg_resources
 
 __all__ = [
     "AIOPartyClient",
@@ -56,47 +51,26 @@ from .pretty.table import write_acs
 from .prim import ContractData, DazlError, FrozenDict as frozendict, Party
 from .util.logging import setup_default_logger
 
+# Large Protobuf message support
+
+# These methods are undocumented, but is required to read even moderately sized DALF's.
 try:
-    # This method is undocumented, but is required to read large size of model files when using
-    # the C++ implementation.
+    # For Protobuf libraries version 4 or later (using upb)
+    # noinspection PyPackageRequirements,PyUnresolvedReferences,PyProtectedMember
+    from google._upb._message import SetAllowOversizeProtos
+
+    SetAllowOversizeProtos(True)
+except ImportError:
+    pass
+
+try:
+    # For Protobuf libraries version 3 or earlier (using upb)
     # noinspection PyPackageRequirements,PyUnresolvedReferences,PyProtectedMember
     from google.protobuf.pyext import _message
 
     _message.SetAllowOversizeProtos(True)
-
 except ImportError:
-    # ImportError for the Protobuf libraries is likely fatal, but this would not be the most helpful
-    # place to throw an ImportError.
     pass
 
 
-def _get_version() -> str:
-    """
-    Used to make the version of this library easily accessible programmatically.
-    Two techniques are tried:
-     1. Try to read it from the current package definition. This is what is used
-        when trying to look up version information if dazl is installed via a
-        wheel file.
-     2. Use the value from the local pyproject.toml file (this is used when
-        running dazl from source).
-    """
-    try:
-        return pkg_resources.require("dazl")[0].version
-    except pkg_resources.DistributionNotFound:
-        pass
-    except Exception:
-        return "unknown"
-
-    try:
-        config = ConfigParser()
-        config.read(Path(__file__).parent.parent / "pyproject.toml")
-        if "tool.poetry" in config:
-            poetry_section = config["tool.poetry"]
-            return literal_eval(poetry_section["version"])
-    except Exception:  # noqa
-        pass
-
-    return "unknown"
-
-
-__version__ = _get_version()
+__version__ = "7.10.0"
