@@ -1,6 +1,7 @@
 # Copyright (c) 2017-2022 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
 # SPDX-License-Identifier: Apache-2.0
-from asyncio import sleep
+
+from asyncio import CancelledError, sleep
 from datetime import datetime, timedelta
 from typing import Awaitable, Callable, TypeVar
 
@@ -17,9 +18,15 @@ def is_retryable_exception(ex: Exception) -> bool:
 
     # TODO: Expand on these categories as documented here:
     #  https://docs.daml.com/app-dev/grpc/error-codes.html
-    return ex.code() == StatusCode.FAILED_PRECONDITION and ex.details().startswith(
+    status = ex.code()
+    if status == StatusCode.UNAVAILABLE:
+        return True
+    elif status == StatusCode.FAILED_PRECONDITION and ex.details().startswith(
         "PARTY_ALLOCATION_WITHOUT_CONNECTED_DOMAIN"
-    )
+    ):
+        return True
+
+    return False
 
 
 async def retry(
@@ -33,8 +40,10 @@ async def retry(
     while True:
         try:
             return await fn()
+        except CancelledError:
+            raise
         except Exception as ex:
-            if (datetime.now() - start) <= timeout and retry_safe_ex(ex):
+            if (datetime.utcnow() - start) <= timeout and retry_safe_ex(ex):
                 # retry in increasing intervals according to the Fibonacci sequence;
                 # exponential backoff generally makes us wait too long
                 a, b = b, a + b
